@@ -32,11 +32,16 @@ class SwiftStorage(Storage):
     tenant_name = setting('SWIFT_TENANT_NAME')
     container_name = setting('SWIFT_CONTAINER_NAME')
     auto_create_container = setting('SWIFT_AUTO_CREATE_CONTAINER', False)
+    auto_create_container_public = setting(
+        'SWIFT_AUTO_CREATE_CONTAINER_PUBLIC', False)
     auto_base_url = setting('SWIFT_AUTO_BASE_URL', True)
     override_base_url = setting('SWIFT_BASE_URL')
     use_temp_urls = setting('SWIFT_USE_TEMP_URLS', False)
     temp_url_key = setting('SWIFT_TEMP_URL_KEY')
     temp_url_duration = setting('SWIFT_TEMP_URL_DURATION', 30*60)
+    auth_token_duration = setting('SWIFT_AUTH_TOKEN_DURATION', 60*60*23)
+    _token_creation_time = 0
+    _token = ''
 
     def __init__(self):
         self.last_headers_name = None
@@ -58,10 +63,14 @@ class SwiftStorage(Storage):
                                        self.container_name,
                                        http_conn=self.http_conn)
         except swiftclient.ClientException:
+            headers = {}
             if self.auto_create_container:
+                if self.auto_create_container_public:
+                    headers['X-Container-Read'] = '.r:*'
                 swiftclient.put_container(self.storage_url, self.token,
                                           self.container_name,
-                                          http_conn=self.http_conn)
+                                          http_conn=self.http_conn,
+                                          headers=headers)
             else:
                 raise ImproperlyConfigured(
                     "Container %s does not exist." % self.container_name)
@@ -87,6 +96,24 @@ class SwiftStorage(Storage):
         else:
             self.base_url = self.override_base_url
 
+    def get_token(self):
+        if time() - self._token_creation_time >= self.auth_token_duration:
+            new_token = swiftclient.get_auth(
+                self.api_auth_url,
+                self.api_username,
+                self.api_key,
+                auth_version=self.auth_version,
+                os_options={"tenant_name": self.tenant_name},
+            )[1]
+            self.token = new_token
+        return self._token
+
+    def set_token(self, new_token):
+        self._token_creation_time = time()
+        self._token = new_token
+
+    token = property(get_token, set_token)
+
     def _open(self, name, mode='rb'):
         headers, content = swiftclient.get_object(self.storage_url, self.token,
                                                   self.container_name, name,
@@ -180,3 +207,7 @@ class SwiftStorage(Storage):
             url = url + '?temp_url_sig=%s&temp_url_expires=%s' % (sig, expires)
 
         return url
+
+class StaticSwiftStorage(SwiftStorage):
+    container_name = setting('SWIFT_STATIC_CONTAINER_NAME')
+
